// Dart imports:
import 'dart:async';

// Flutter imports:
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

// Package imports:
import 'package:flutter_screenutil/flutter_screenutil.dart';

// Project imports:
import 'package:zego_uikit/src/components/internal/internal.dart';
import 'package:zego_uikit/src/service/service.dart';

/// type of audio video view background builder
typedef MemberListItemBuilder = Widget Function(
    BuildContext context, Size size, ZegoUIKitUser user, Map extraInfo);

class ZegoMemberList extends StatefulWidget {
  final MemberListItemBuilder? itemBuilder;

  const ZegoMemberList({
    Key? key,
    this.itemBuilder,
  }) : super(key: key);

  @override
  State<ZegoMemberList> createState() => _ZegoCallMemberListState();
}

class _ZegoCallMemberListState extends State<ZegoMemberList> {
  var usersNotifier = ValueNotifier<List<ZegoUIKitUser>>([]);
  StreamSubscription<dynamic>? userListSubscription;

  @override
  void initState() {
    super.initState();

    usersNotifier.value = ZegoUIKit().getAllUsers();
    userListSubscription =
        ZegoUIKit().getUserListStream().listen(onUserListUpdated);
  }

  @override
  void dispose() {
    super.dispose();

    userListSubscription?.cancel();
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        header(),
        ValueListenableBuilder<List<ZegoUIKitUser>>(
          valueListenable: usersNotifier,
          builder: (context, users, child) {
            var itemSize = Size(750.w, 72.h);
            return ListView.builder(
              shrinkWrap: true,
              padding: EdgeInsets.only(top: 48.r),
              itemCount: users.length,
              itemBuilder: (context, index) {
                var user = users[index];
                return widget.itemBuilder?.call(context, itemSize, user, {}) ??
                    listItem(context, itemSize, user);
              },
            );
          },
        ),
      ],
    );
  }

  Widget header() {
    return Row(
      children: [
        GestureDetector(
          onTap: () {
            Navigator.of(context).pop();
          },
          child: SizedBox(
            width: 70.r,
            height: 70.r,
            child: UIKitImage.asset(StyleIconUrls.iconBack),
          ),
        ),
        SizedBox(width: 10.r),
        Text(
          "Member",
          style: TextStyle(
            fontSize: 36.0.r,
            color: const Color(0xffffffff),
            decoration: TextDecoration.none,
          ),
        )
      ],
    );
  }

  Widget listItem(BuildContext context, Size itemSize, ZegoUIKitUser user) {
    return Container(
      margin: EdgeInsets.only(bottom: 36.r),
      child: Row(
        children: [
          SizedBox(width: 36.r),
          memberListNameIcon(user.name),
          SizedBox(width: 20.r),
          memberListItemUserName(user.name),
          const Expanded(child: SizedBox()),
          SizedBox(width: 34.r)
        ],
      ),
    );
  }

  void onUserListUpdated(List<ZegoUIKitUser> users) {
    usersNotifier.value = users;
  }
}

Widget memberListItemUserName(String name) {
  return Text(
    name,
    style: TextStyle(
      fontSize: 32.0.r,
      color: const Color(0xffffffff),
      decoration: TextDecoration.none,
    ),
  );
}

Widget memberListNameIcon(String name) {
  return Container(
    width: 72.r,
    height: 72.r,
    decoration:
        const BoxDecoration(color: Color(0xffDBDDE3), shape: BoxShape.circle),
    child: Center(
      child: Text(
        name.isNotEmpty ? name.characters.first : "",
        style: TextStyle(
          fontSize: 32.0.r,
          color: const Color(0xff222222),
          decoration: TextDecoration.none,
        ),
      ),
    ),
  );
}
